import sys

import requests

from .utils import make_get_json, make_post_json, _resource_url


def _resource_path(resource_path, group):
    return '{}/{}.json'.format(resource_path, group)


def make_path(path):
    return 'statusnet/groups/{}'.format(path)


create = make_post_json('create', make_path('create.json'))

for name, path in (
        ('local_groups', 'list_all.json'),
        ('user_groups', 'list.json'),
):
    setattr(sys.modules[__name__], name, make_get_json(name, make_path(path)))


def make_get_group(name, resource_path):
    def func(server_url, group, *args, **kwargs):
        url = _resource_url(server_url, _resource_path(resource_path, group))
        return requests.get(url, *args, **kwargs).json()
    func.__name__ = name
    return func


def make_post_group(name, resource_path):
    def func(server_url, group, *args, **kwargs):
        url = _resource_url(server_url, _resource_path(resource_path, group))
        return requests.post(url, *args, **kwargs).json()
    func.__name__ = name
    return func


for name, path in (
        ('timeline', 'timeline'),
        ('show', 'show'),
        ('is_member', 'is_member'),
        ('admins', 'admins'),
        ('members', 'membership')
):
    setattr(sys.modules[__name__], name, make_get_group(name, make_path(path)))


for name in ('leave', 'join'):
    setattr(sys.modules[__name__], name, make_post_group(name, make_path(name)))



import sys
from .utils import make_get_json

for name, path in (
        ('friends', 'statuses/friends.json'),
        ('followers', 'statuses/followers.json'),
        ('show', 'users/show.json'),
        ('friends_ids', 'friends/ids.json'),
        ('followers_ids', 'followers/ids.json')
):
    setattr(sys.modules[__name__], name, make_get_json(name, path))



import re

import requests

from .exceptions import ServerURLError

DOMAIN_REGEX = re.compile(r"http(s|)://(www\.|)(.+?)(/.*|)$")


def _api_path(server_url):
    _validate_server_url(server_url)
    if server_url[-1] != '/':
        server_url += '/'
    return server_url + 'api/'


def _validate_server_url(server_url):
    if not DOMAIN_REGEX.match(server_url):
        raise ServerURLError(server_url)


def _resource_url(server_url, resource_path):
    return _api_path(server_url) + resource_path


def make_get_json(name, resource_path):
    def func(server_url, *args, **kwargs):
        return requests.get(_resource_url(server_url, resource_path), *args, **kwargs).json()
    func.__name__ = name
    return func


def make_post_json(name, resource_path):
    def func(server_url, *args, **kwargs):
        return requests.post(_resource_url(server_url, resource_path), *args, **kwargs).json()
    func.__name__ = name
    return func



from requests import get

from .utils import _resource_url

def search(server_url, query, *args, **kwargs):
    params = kwargs.get('params', {})
    params['q'] = query
    return get(_resource_url(server_url, 'search.json'), params=params).json()



class ServerURLError(Exception):
    def __init__(self, server_url):
        self.server_url = server_url
        super().__init__(Exception)

    def __repr__(self):
        return 'ServerURLError(%r)' % self.server_url

    def __str__(self):
        return 'Invalid server URL %s' % self.server_url



import sys
from .utils import make_get_json, make_post_json

def make_path(path):
    return 'account/{}.json'.format(path)


verify_credentials = make_get_json('verify_credentials', make_path('verify_credentials'))


for name in (
        'update_profile_image',
        'update_profile',
        'register'
):
    setattr(sys.modules[__name__], name, make_post_json(name, make_path(name)))



import sys
from .utils import make_get_json

def make_path(path):
    return 'statuses/{}.json'.format(path)


for name, path in (
        ('public', 'public_timeline'),
        ('home', 'home_timeline'),
        ('friends', 'friends_timeline'),
        ('user', 'user_timeline'),
        ('mentions', 'mentions'),
        ('replies', 'replies')
):
    setattr(sys.modules[__name__], name, make_get_json(name, make_path(path)))



import sys

import requests

from .utils import _resource_url, make_get_json, make_post_json


def add_id(resource_path, item_id):
    return '{}/{}.json'.format(resource_path, item_id)


def favorites_path(path):
    return 'favorites/{}'.format(path)


def statuses_path(path):
    return 'statuses/{}'.format(path)


def make_get_status(name, resource_path):
    def func(server_url, status_id, *args, **kwargs):
        url = _resource_url(server_url, add_id(resource_path, status_id))
        return requests.get(url, *args, **kwargs).json()
    func.__name__ = name
    return func


def make_post_status(name, resource_path):
    def func(server_url, status_id, *args, **kwargs):
        url = _resource_url(server_url, add_id(resource_path, status_id))
        return requests.post(url, *args, **kwargs).json()
    func.__name__ = name
    return func


favorites = make_get_json('favorites', 'favorites.json')
update = make_post_json('update', statuses_path('update.json'))
show = make_get_status('show', statuses_path('show'))


for name, path in (
        ('destroy', 'destroy'),
        ('repeat', 'retweet')
):
    setattr(sys.modules[__name__], name, make_post_status(name, statuses_path(path)))


for name, path in (
        ('unfavorite', 'destroy'),
        ('favorite', 'create')
):
    setattr(sys.modules[__name__], name, make_post_status(name, favorites_path(path)))


def conversation(server_url, conversation_id, *args, **kwargs):
    url = add_id(_resource_url(server_url, 'statusnet/conversation'), conversation_id)
    return requests.get(url, *args, **kwargs).json()



import sys
from .utils import make_get_json, make_post_json


def make_path(path):
    prefix = 'direct_messages'
    if path:
        return prefix + '/{}.json'.format(path)
    return prefix + '.json'


for name, path in (
        ('received', ''),
        ('sent', 'sent'),
):
    setattr(sys.modules[__name__], name, make_get_json(name, make_path(path)))


new = make_post_json('new', make_path('new'))



import sys
from .utils import make_post_json

def make_path(path):
    return 'blocks/{}.json'.format(path)


for name in ('create', 'destroy'):
    setattr(sys.modules[__name__], name, make_post_json(name, make_path(name)))



import sys

from .utils import make_get_json, make_post_json


def make_path(path):
    return 'friendships/{}.json'.format(path)


for name in ('exists', 'show') :
    setattr(sys.modules[__name__], name, make_get_json(name, make_path(name)))

for name in ('create', 'destroy'):
    setattr(sys.modules[__name__], name, make_post_json(name, make_path(name)))



import sys
from urllib.parse import parse_qs

import requests
from requests_oauthlib import OAuth1

from .utils import _resource_url


def make_oauth_object(consumer_key, consumer_secret, resource_owner_key=None,
                      resource_owner_secret=None, secret_key=None):
    return OAuth1(client_key=consumer_key, client_secret=consumer_secret,
                  resource_owner_key=resource_owner_key,
                  resource_owner_secret=resource_owner_secret,
                  verifier=secret_key)


def _parse_response(response):
    creds = parse_qs(response.text)
    res = {}
    res['resource_owner_key'] = creds['oauth_token'][0]
    res['resource_owner_secret'] = creds['oauth_token_secret'][0]
    return res


def _default_oauth_callback(data):
    if not data.get('oauth_callback'):
        data.update({'oauth_callback': 'oob'})
    return data


def _make_post_oauth(name, resource_path):
    def func(server_url, *args, **kwargs):
        data = kwargs.get('data')
        if data:
            kwargs['data'] = _default_oauth_callback(data)
        resp = requests.post(_resource_url(server_url, resource_path), *args, **kwargs)
        return _parse_response(resp)
    func.__name__ = name
    return func


def make_path(path):
    return 'oauth/{}'.format(path)


for name in ('request_token', 'access_token'):
    setattr(sys.modules[__name__], name, _make_post_oauth(name, make_path(name)))


def authorize_url(server_url, resource_owner_key):
    return _resource_url(server_url, 'oauth/authorize') +\
        '?oauth_token=' + resource_owner_key
